(*

                     Inertia Realtime 3D Rendering Engine
            Copyright (c) 1997, Alex Chalfin. All Rights Reserved.
           DISTRIBUTION OF THIS SOURCE CODE IS STRICTLY PROHIBITED

*)
Unit Matrix;
{$I DEFINE.INC}

Interface

Uses
  Misc,
  Types;

Procedure MatrixCopy(Var Source, Dest : Matrix4x3);
Procedure InitMatrix(Var M : Matrix4x3);
Procedure SetRotationMatrix(Var M : Matrix4x3; xa, ya, za : Single);
Procedure SetScaleMatrix(Var M : Matrix4x3; sx, sy, sz : Single);
Procedure MatrixMul(Var Result, A, B : Matrix4x3);
Procedure MatrixOrthoNormalize(Var M : Matrix4x3);
Procedure AxisRotate(Var M : Matrix4x3; angle, x, y, z : Single);

Implementation

Procedure MatrixCopy(Var Source, Dest : Matrix4x3);

Begin
  Move(Source, Dest, sizeof(Matrix4x3));
End;

Procedure InitMatrix(Var M : Matrix4x3);

Begin
  FillChar(M, sizeof(Matrix4x3), 0);

  M[0] := 1.0; M[4] := 1.0; M[8] := 1.0;
End;

Procedure SetScaleMatrix(Var M : Matrix4x3; sx, sy, sz : Single);

Begin
  FillChar(M, sizeof(Matrix4x3), 0);
  M[0] := sx; M[4] := sy; M[8] := sz;
End;

{$L MATRIX.OBJ}
Procedure MatrixMul(Var Result, A, B : Matrix4x3); External;

Procedure SetRotationMatrix(Var M : Matrix4x3; xa, ya, za : Single);

Var
  sx, sy, sz : Single;
  cx, cy, cz : Single;

Begin
  sx := sin(xa); sy := sin(ya); sz := sin(za);
  cx := cos(xa); cy := cos(ya); cz := cos(za);

  M[0] := (cz * cy) + (sx * sz * sy);
  M[3] := (cy * -sz) + (cz * sx * sy);
  M[6] := (cx * sy);
  M[1] := (sz * cx);
  M[4] := (cz * cx);
  M[7] := (-sx);
  M[2] := (-sy * cz) + (sz * sx * cy);
  M[5] := (sz * sy) + (cz * sx * cy);
  M[8] := (cx * cy);
End;


Procedure MatrixOrthoNormalize(Var M : Matrix4x3);

Var
  dot1, dot2, len : Single;

Begin
  dot1 := M[0] * M[1] + M[3] * M[4] + M[6] * M[7];
  dot2 := M[0] * M[2] + M[3] * M[5] + M[6] * M[8];

  M[0] := M[0] - dot1 * M[1];
  M[3] := M[3] - dot1 * M[4];
  M[6] := M[6] - dot1 * M[7];
  M[0] := M[0] - dot2 * M[2];
  M[3] := M[3] - dot2 * M[5];
  M[6] := M[6] - dot2 * M[8];

  len := 1.0 / sqrt(sqr(M[0]) + sqr(M[3]) + sqr(M[6]));

  M[0] := M[0] * len;
  M[3] := M[3] * len;
  M[6] := M[6] * len;

  dot1 := M[1] * M[0] + M[4] * M[3] + M[7] * M[6];
  dot2 := M[1] * M[2] + M[4] * M[5] + M[7] * M[8];

  M[1] := M[1] - dot1 * M[0];
  M[4] := M[4] - dot1 * M[3];
  M[7] := M[7] - dot1 * M[6];
  M[1] := M[1] - dot2 * M[2];
  M[4] := M[4] - dot2 * M[5];
  M[7] := M[7] - dot2 * M[8];

  len := 1.0 / sqrt(SQR(M[1]) + SQR(M[4]) + SQR(M[7]));
  M[1] := M[1] * len;
  M[4] := M[4] * len;
  M[7] := M[7] * len;

  M[2] := M[3] * M[7] - M[6] * M[4];
  M[5] := M[6] * M[1] - M[0] * M[7];
  M[8] := M[0] * M[4] - M[3] * M[1];
End;

Procedure AxisRotate(Var M : Matrix4x3; angle, x, y, z : Single);

Var
  cosa, sina : Single;
  dot, dx, dy, dz : Single;
  rx, ry, rz : Single;
  cx, cy, cz : Single;
  i : Longint;

Begin
  cosa := cos(angle);
  sina := sin(angle);

  For i := 0 to 2 do
    Begin
      dot := (x * M[i+0] + y * M[i+3] + z * M[i+6]);

      dx := dot * x * (1.0 - cosa);
      dy := dot * y * (1.0 - cosa);
      dz := dot * z * (1.0 - cosa);

      rx := M[i+0] * cosa;
      ry := M[i+3] * cosa;
      rz := M[i+6] * cosa;

      cx := (y * M[i+6] - z * M[i+3]) * sina;
      cy := (z * M[i+0] - x * M[i+6]) * sina;
      cz := (x * M[i+3] - y * M[i+0]) * sina;

      M[i+0] := rx + dx + cx;
      M[i+3] := ry + dy + cy;
      M[i+6] := rz + dz + cz;
    End;
End;


End.