(*

                     Inertia Realtime 3D Rendering Engine
            Copyright (c) 1997, Alex Chalfin. All Rights Reserved.
           DISTRIBUTION OF THIS SOURCE CODE IS STRICTLY PROHIBITED

*)

Unit Pool;
{$R-}
{$S-}
{$Q-}

Interface

Uses
  Types;

Const
  POOL_NODE_COUNT = 32;
  POOL_NODE_MASK = POOL_NODE_COUNT - 1;

Type
  pVertexNode = ^VertexNode;
  VertexNode = Record
    VList : Array[0..POOL_NODE_COUNT - 1] of Vertex;
    Next  : pVertexNode;
  End;

  pPolygonNode = ^PolygonNode;
  PolygonNode = Record
    PList : Array[0..POOL_NODE_COUNT - 1] of Triangle;
    Next  : pPolygonNode;
  End;

Function GrabVertexFromPool :  pVertex;
Function GrabPolygonFromPool : pTriangle;
Procedure ResetVertexPool;
Procedure ResetPolygonPool;
Procedure InitVertexPool;
Procedure InitPolygonPool;
Procedure FreePolygonPool;
Procedure FreeVertexPool;

Implementation

Var
  VertexPool : Record
                 Count      : Longint;
                 Allocated  : Longint;
                 Traverse   : pVertexNode;
                 VertexList : pVertexNode;
               End;

  PolygonPool : Record
                 Count      : Longint;
                 Allocated  : Longint;
                 Traverse   : pPolygonNode;
                 PolygonList : pPolygonNode;
               End;



Function GrabVertexFromPool : pVertex;

Var
  TempPtr : pVertexNode;

Begin
  if ((VertexPool.Count + 1) >= VertexPool.Allocated)
    Then Begin
      New(TempPtr);

      if (TempPtr = NIL)
        Then Begin
          GrabVertexFromPool := Nil;
          Exit;
        End;

      TempPtr^.Next := NIL;
      if (VertexPool.VertexList = NIL)
        Then Begin
          VertexPool.VertexList := TempPtr;
          VertexPool.Traverse   := TempPtr;
        End
        Else Begin
          VertexPool.Traverse^.Next := TempPtr;
          VertexPool.Traverse       := VertexPool.Traverse^.Next;
        End;

      Inc(VertexPool.Allocated, POOL_NODE_COUNT);
    End
    Else Begin
      if (((VertexPool.Count + 1) And POOL_NODE_MASK) < ((VertexPool.Count) And POOL_NODE_MASK))
        Then VertexPool.Traverse := VertexPool.Traverse^.Next;
    End;

  Inc(VertexPool.Count);
  GrabVertexFromPool := @VertexPool.Traverse^.VList[(VertexPool.Count - 1) And POOL_NODE_MASK];
End;


Function GrabPolygonFromPool : pTriangle;

Var
  TempPtr : pPolygonNode;

Begin
  if ((PolygonPool.Count + 1) >= PolygonPool.Allocated)
    Then Begin
      New(TempPtr);

      if (TempPtr = NIL)
        Then Begin
          GrabPolygonFromPool := Nil;
          Exit;
        End;

      TempPtr^.Next := NIL;
      if (PolygonPool.PolygonList = NIL)
        Then Begin
          PolygonPool.PolygonList := TempPtr;
          PolygonPool.Traverse    := TempPtr;
        End
        Else Begin
          PolygonPool.Traverse^.Next := TempPtr;
          PolygonPool.Traverse       := PolygonPool.Traverse^.Next;
        End;

      Inc(PolygonPool.Allocated, POOL_NODE_COUNT);
    End
    Else Begin
      if (((PolygonPool.Count + 1) And POOL_NODE_MASK) < ((PolygonPool.Count) And POOL_NODE_MASK))
        Then PolygonPool.Traverse := PolygonPool.Traverse^.Next;
    End;

  Inc(PolygonPool.Count);
  GrabPolygonFromPool := @PolygonPool.Traverse^.PList[(PolygonPool.Count - 1) And POOL_NODE_MASK];
End;

Procedure ResetVertexPool;

Begin
  VertexPool.Count    := 0;
  VertexPool.Traverse := VertexPool.VertexList;
End;

Procedure ResetPolygonPool;

Begin
  PolygonPool.Count    := 0;
  PolygonPool.Traverse := PolygonPool.PolygonList;
End;

Procedure InitVertexPool;

Begin
  VertexPool.Count      := 0;
  VertexPool.Allocated  := 0;
  VertexPool.VertexList := NIL;
  VertexPool.Traverse   := NIL;
End;

Procedure InitPolygonPool;

Begin
  PolygonPool.Count       := 0;
  PolygonPool.Allocated   := 0;
  PolygonPool.Traverse    := NIL;
  PolygonPool.PolygonList := NIL;
End;

Procedure FreePolygonPool;

Var
  TempPtr : pPolygonNode;

Begin
  PolygonPool.Allocated := 0;
  PolygonPool.Count     := 0;

  PolygonPool.Traverse := PolygonPool.PolygonList;

  while (PolygonPool.Traverse <> NIL) do
    Begin
      TempPtr := PolygonPool.Traverse;
      PolygonPool.Traverse := PolygonPool.Traverse^.Next;
      Dispose(TempPtr);
    End;

  PolygonPool.PolygonList := NIL;
End;

Procedure FreeVertexPool;

Var
  TempPtr : pVertexNode;

Begin
  VertexPool.Allocated := 0;
  VertexPool.Count     := 0;

  VertexPool.Traverse := VertexPool.VertexList;

  while (VertexPool.Traverse <> NIL) do
    Begin
      TempPtr := VertexPool.Traverse;
      VertexPool.Traverse := VertexPool.Traverse^.Next;
      Dispose(TempPtr);
    End;

  VertexPool.VertexList := NIL;
End;

End.
